var n={7899:n=>{n.exports='/*\nauthor: Patricio Gonzalez Vivo\ndescription: Returns a circle-shaped SDF.\nuse: circleSDF(vec2 st[, vec2 center])\noptions:\n  CIRCLESDF_FNC(POS_UV) : function used to calculate the SDF, defaults to GLSL length function, use lengthSq for a different slope\nlicense: |\n  Copyright (c) 2017 Patricio Gonzalez Vivo. All rights reserved.\n  Distributed under BSD 3-clause "New" or "Revised" License. See LICENSE file at\n  https://github.com/patriciogonzalezvivo/PixelSpiritDeck\n*/\n\n#ifndef CIRCLESDF_FNC\n#define CIRCLESDF_FNC(POS_UV) length(POS_UV)\n#endif\n\n#ifndef FNC_CIRCLESDF\n#define FNC_CIRCLESDF\n\nfloat circleSDF(in vec2 st, in vec2 center) {\n    return CIRCLESDF_FNC(st - center) * 2.;\n}\n\nfloat circleSDF(in vec2 st) {\n    return circleSDF(st, vec2(.5));\n}\n\n#endif\n'},2092:n=>{n.exports='/*\nauthor: Patricio Gonzalez Vivo\ndescription: Returns a rectangular SDF\nuse: rectSDF(<vec2> st, <vec2> size)\nlicense: |\n  Copyright (c) 2017 Patricio Gonzalez Vivo. All rights reserved.\n  Distributed under BSD 3-clause "New" or "Revised" License. See LICENSE file at\n  https://github.com/patriciogonzalezvivo/PixelSpiritDeck\n*/\n\n#ifndef FNC_RECTSDF\n#define FNC_RECTSDF\nfloat rectSDF(in vec2 st, in vec2 s) {\n    st = st * 2. - 1.;\n    return max( abs(st.x / s.x),\n                abs(st.y / s.y) );\n}\n#endif\n\n\n/*\nauthor: Patricio Gonzalez Vivo\ndescription: Returns a cross-shaped SDF\nuse: crossSDF(<vec2> st, size s)\nlicense: |\n  Copyright (c) 2017 Patricio Gonzalez Vivo. All rights reserved.\n  Distributed under BSD 3-clause "New" or "Revised" License. See LICENSE file at\n  https://github.com/patriciogonzalezvivo/PixelSpiritDeck\n*/\n\n#ifndef FNC_CROSSSDF\n#define FNC_CROSSSDF\nfloat crossSDF(in vec2 st, in float s) {\n    vec2 size = vec2(.25, s);\n    return min(rectSDF(st.xy, size.xy),\n               rectSDF(st.xy, size.yx));\n}\n#endif\n'},9023:n=>{n.exports='/*\nauthor: Patricio Gonzalez Vivo\ndescription: Returns a flower shaped SDF\nuse: flowerSDF(<vec2> st, <int> n_sides)\nlicense: |\n  Copyright (c) 2017 Patricio Gonzalez Vivo. All rights reserved.\n  Distributed under BSD 3-clause "New" or "Revised" License. See LICENSE file at\n  https://github.com/patriciogonzalezvivo/PixelSpiritDeck\n*/\n\n#ifndef FNC_FLOWERSDF\n#define FNC_FLOWERSDF\nfloat flowerSDF(vec2 st, int N) {\n    st = st * 2.0 - 1.0;\n    float r = length(st) * 2.0;\n    float a = atan(st.y, st.x);\n    float v = float(N) * 0.5;\n    return 1.0 - (abs(cos(a * v)) *  0.5 + 0.5) / r;\n}\n#endif'},6881:n=>{n.exports='/*\nauthor: Patricio Gonzalez Vivo\ndescription: Returns a heart shaped SDF\nuse: heartSDF(<vec2> st)\nlicense: |\n  Copyright (c) 2017 Patricio Gonzalez Vivo. All rights reserved.\n  Distributed under BSD 3-clause "New" or "Revised" License. See LICENSE file at\n  https://github.com/patriciogonzalezvivo/PixelSpiritDeck\n*/\n\n#ifndef FNC_HEARTSDF\n#define FNC_HEARTSDF\nfloat heartSDF(vec2 st) {\n    st -= vec2(0.5, 0.8);\n    float r = length(st) * 5.0;\n    st = normalize(st);\n    return r - ((st.y * pow(abs(st.x), 0.67)) / (st.y + 1.5) - (2.0) * st.y + 1.26);\n}\n#endif'},9455:n=>{n.exports='/*\\\nauthor: Patricio Gonzalez Vivo\ndescription: Returns a hexagon-shaped SDF\nuse: hexSDF(<vec2> st)\nlicense: |\n  Copyright (c) 2017 Patricio Gonzalez Vivo. All rights reserved.\n  Distributed under BSD 3-clause "New" or "Revised" License. See LICENSE file at\n  https://github.com/patriciogonzalezvivo/PixelSpiritDeck\n*/\n\n#ifndef FNC_HEXSDF\n#define FNC_HEXSDF\nfloat hexSDF(in vec2 st) {\n    st = abs(st * 2. - 1.);\n    return max(abs(st.y), st.x * .866025 + st.y * .5);\n}\n#endif\n'},6223:n=>{n.exports='#ifndef QTR_PI\n#define QTR_PI 0.78539816339\n#endif\n#ifndef HALF_PI\n#define HALF_PI 1.5707963267948966192313216916398\n#endif\n#ifndef PI\n#define PI 3.1415926535897932384626433832795\n#endif\n#ifndef TWO_PI\n#define TWO_PI 6.2831853071795864769252867665590\n#endif\n#ifndef TAU\n#define TAU 6.2831853071795864769252867665590\n#endif\n#ifndef ONE_OVER_PI\n#define ONE_OVER_PI 0.31830988618\n#endif\n#ifndef PHI\n#define PHI 1.618033988749894848204586834\n#endif\n#ifndef EPSILON\n#define EPSILON 0.0000001\n#endif\n#ifndef GOLDEN_RATIO\n#define GOLDEN_RATIO 1.6180339887\n#endif\n#ifndef GOLDEN_ANGLE // (3.-sqrt(5.0))*PI radians\n#define GOLDEN_ANGLE 2.39996323\n#endif\n\n\n/*\nauthor: Patricio Gonzalez Vivo\ndescription: Returns a sdf for a regular polygon with V sides.\nuse: polySDF(<vec2> st, int V)\nlicense: |\n  Copyright (c) 2017 Patricio Gonzalez Vivo. All rights reserved.\n  Distributed under BSD 3-clause "New" or "Revised" License. See LICENSE file at\n  https://github.com/patriciogonzalezvivo/PixelSpiritDeck\n*/\n\n#ifndef FNC_POLYSDF\n#define FNC_POLYSDF\nfloat polySDF(in vec2 st, in int V) {\n    st = st * 2. - 1.;\n    float a = atan(st.x, st.y) + PI;\n    float r = length(st);\n    float v = TAU / float(V);\n    return cos(floor(.5 + a / v) * v - a ) * r;\n}\n#endif\n'},122:n=>{n.exports='#ifndef QTR_PI\n#define QTR_PI 0.78539816339\n#endif\n#ifndef HALF_PI\n#define HALF_PI 1.5707963267948966192313216916398\n#endif\n#ifndef PI\n#define PI 3.1415926535897932384626433832795\n#endif\n#ifndef TWO_PI\n#define TWO_PI 6.2831853071795864769252867665590\n#endif\n#ifndef TAU\n#define TAU 6.2831853071795864769252867665590\n#endif\n#ifndef ONE_OVER_PI\n#define ONE_OVER_PI 0.31830988618\n#endif\n#ifndef PHI\n#define PHI 1.618033988749894848204586834\n#endif\n#ifndef EPSILON\n#define EPSILON 0.0000001\n#endif\n#ifndef GOLDEN_RATIO\n#define GOLDEN_RATIO 1.6180339887\n#endif\n#ifndef GOLDEN_ANGLE // (3.-sqrt(5.0))*PI radians\n#define GOLDEN_ANGLE 2.39996323\n#endif\n\n\n/*\nauthor: Patricio Gonzalez Vivo\ndescription: Returns a sdf for rays with N branches\nuse: raysSDF(<vec2> st, <int> N)\nlicense: |\n  Copyright (c) 2017 Patricio Gonzalez Vivo. All rights reserved.\n  Distributed under BSD 3-clause "New" or "Revised" License. See LICENSE file at\n  https://github.com/patriciogonzalezvivo/PixelSpiritDeck\n*/\n\n#ifndef FNC_RAYSSDF\n#define FNC_RAYSSDF\nfloat raysSDF(in vec2 st, in int N) {\n    st -= .5;\n    return fract(atan(st.y, st.x) / TAU * float(N));\n}\n#endif\n'},2704:n=>{n.exports='/*\nauthor: Patricio Gonzalez Vivo\ndescription: Returns a rectangular SDF\nuse: rectSDF(<vec2> st, <vec2> size)\nlicense: |\n  Copyright (c) 2017 Patricio Gonzalez Vivo. All rights reserved.\n  Distributed under BSD 3-clause "New" or "Revised" License. See LICENSE file at\n  https://github.com/patriciogonzalezvivo/PixelSpiritDeck\n*/\n\n#ifndef FNC_RECTSDF\n#define FNC_RECTSDF\nfloat rectSDF(in vec2 st, in vec2 s) {\n    st = st * 2. - 1.;\n    return max( abs(st.x / s.x),\n                abs(st.y / s.y) );\n}\n#endif\n'},4260:n=>{n.exports='/*\nauthor: Patricio Gonzalez Vivo\ndescription: Returns a spiral SDF\nuse: spiralSDF(<vec2> st, <float> turns)\nlicense: |\n  Copyright (c) 2017 Patricio Gonzalez Vivo. All rights reserved.\n  Distributed under BSD 3-clause "New" or "Revised" License. See LICENSE file at\n  https://github.com/patriciogonzalezvivo/PixelSpiritDeck\n*/\n\n#ifndef FNC_SPIRALSDF\n#define FNC_SPIRALSDF\nfloat spiralSDF(vec2 st, float t) {\n    st -= 0.5;\n    float r = dot(st, st);\n    float a = atan(st.y, st.x);\n    return abs(sin(fract(log(r) * t + a * 0.159)));\n}\n#endif'},3433:n=>{n.exports='#ifndef QTR_PI\n#define QTR_PI 0.78539816339\n#endif\n#ifndef HALF_PI\n#define HALF_PI 1.5707963267948966192313216916398\n#endif\n#ifndef PI\n#define PI 3.1415926535897932384626433832795\n#endif\n#ifndef TWO_PI\n#define TWO_PI 6.2831853071795864769252867665590\n#endif\n#ifndef TAU\n#define TAU 6.2831853071795864769252867665590\n#endif\n#ifndef ONE_OVER_PI\n#define ONE_OVER_PI 0.31830988618\n#endif\n#ifndef PHI\n#define PHI 1.618033988749894848204586834\n#endif\n#ifndef EPSILON\n#define EPSILON 0.0000001\n#endif\n#ifndef GOLDEN_RATIO\n#define GOLDEN_RATIO 1.6180339887\n#endif\n#ifndef GOLDEN_ANGLE // (3.-sqrt(5.0))*PI radians\n#define GOLDEN_ANGLE 2.39996323\n#endif\n\n\n/*\nauthor: Patricio Gonzalez Vivo\ndescription: Returns a star-shaped sdf with V branches\nuse: starSDF(<vec2> st, <int> V, <float> scale)\nlicense: |\n  Copyright (c) 2017 Patricio Gonzalez Vivo. All rights reserved.\n  Distributed under BSD 3-clause "New" or "Revised" License. See LICENSE file at\n  https://github.com/patriciogonzalezvivo/PixelSpiritDeck\n*/\n\n#ifndef FNC_STARSDF\n#define FNC_STARSDF\nfloat starSDF(in vec2 st, in int V, in float s) {\n    st = st * 4. - 2.;\n    float a = atan(st.y, st.x) / TAU;\n    float seg = a * float(V);\n    a = ((floor(seg) + .5) / float(V) +\n        mix(s, -s, step(.5, fract(seg))))\n        * TAU;\n    return abs(dot(vec2(cos(a), sin(a)),\n                   st));\n}\n#endif\n'},5866:n=>{n.exports='/*\nauthor: Patricio Gonzalez Vivo\ndescription: Returns a triangle-shaped sdf\nuse: triSDF(<vec2> st)\nlicense: |\n  Copyright (c) 2017 Patricio Gonzalez Vivo. All rights reserved.\n  Distributed under BSD 3-clause "New" or "Revised" License. See LICENSE file at\n  https://github.com/patriciogonzalezvivo/PixelSpiritDeck\n*/\n\n#ifndef FNC_TRISDF\n#define FNC_TRISDF\nfloat triSDF(in vec2 st) {\n    st = (st * 2. - 1.) * 2.;\n    return max(abs(st.x) * .866025 + st.y * .5, -st.y * .5);\n}\n#endif\n'},2838:n=>{n.exports='/*\nauthor: Patricio Gonzalez Vivo\ndescription: Returns a circle-shaped SDF.\nuse: circleSDF(vec2 st[, vec2 center])\noptions:\n  CIRCLESDF_FNC(POS_UV) : function used to calculate the SDF, defaults to GLSL length function, use lengthSq for a different slope\nlicense: |\n  Copyright (c) 2017 Patricio Gonzalez Vivo. All rights reserved.\n  Distributed under BSD 3-clause "New" or "Revised" License. See LICENSE file at\n  https://github.com/patriciogonzalezvivo/PixelSpiritDeck\n*/\n\n#ifndef CIRCLESDF_FNC\n#define CIRCLESDF_FNC(POS_UV) length(POS_UV)\n#endif\n\n#ifndef FNC_CIRCLESDF\n#define FNC_CIRCLESDF\n\nfloat circleSDF(in vec2 st, in vec2 center) {\n    return CIRCLESDF_FNC(st - center) * 2.;\n}\n\nfloat circleSDF(in vec2 st) {\n    return circleSDF(st, vec2(.5));\n}\n\n#endif\n\n\n/*\nauthor: Patricio Gonzalez Vivo\ndescription: Returns an almond-shaped sdf\nuse: vesicaSDF(<vec2> st, <float> w)\nlicense: |\n  Copyright (c) 2017 Patricio Gonzalez Vivo. All rights reserved.\n  Distributed under BSD 3-clause "New" or "Revised" License. See LICENSE file at\n  https://github.com/patriciogonzalezvivo/PixelSpiritDeck\n*/\n\n#ifndef FNC_VESICASDF\n#define FNC_VESICASDF\nfloat vesicaSDF(in vec2 st, in float w) {\n    vec2 offset = vec2(w*.5,0.);\n    return max( circleSDF(st-offset),\n                circleSDF(st+offset));\n}\n#endif\n'}},e={};function i(t){var s=e[t];if(void 0!==s)return s.exports;var r=e[t]={exports:{}};return n[t](r,r.exports,i),r.exports}i.n=n=>{var e=n&&n.__esModule?()=>n.default:()=>n;return i.d(e,{a:e}),e},i.d=(n,e)=>{for(var t in e)i.o(e,t)&&!i.o(n,t)&&Object.defineProperty(n,t,{enumerable:!0,get:e[t]})},i.o=(n,e)=>Object.prototype.hasOwnProperty.call(n,e);var t={};(()=>{i.d(t,{Z:()=>L});var n=i(7899),e=i.n(n),s=i(9023),r=i.n(s),o=i(9455),a=i.n(o),f=i(122),c=i.n(f),d=i(4260),l=i.n(d),S=i(5866),D=i.n(S),v=i(2092),F=i.n(v),u=i(6881),p=i.n(u),h=i(6223),N=i.n(h),P=i(2704),C=i.n(P),z=i(3433),E=i.n(z),_=i(2838),I=i.n(_);const L={circleSDF:e(),flowerSDF:r(),hexSDF:a(),raysSDF:c(),spiralSDF:l(),triSDF:D(),crossSDF:F(),heartSDF:p(),polySDF:N(),rectSDF:C(),starSDF:E(),vesicaSDF:I()}})();var s=t.Z;export{s as default};